@@ -3,7 +3,7 @@
  * Interface used to send events.
  *****************************************************************************
  * Copyright (C) 2007 VLC authors and VideoLAN
- * $Id: 328e2bd20e157da31cbad80c4c3e0078c7298de7 $
+ * $Id: 229f4f677029763a837558932dad3e47b283e61c $
  *
  * Authors: Pierre d'Herbemont
  *
@@ -70,8 +70,6 @@
  * {
  *        ...
  *        vlc_event_manager_init( &p_self->p_event_manager, p_self, p_a_libvlc_object );
- *        vlc_event_manager_register_event_type(p_self->p_event_manager,
- *                vlc_MyCoolObjectDidSomething, p_e)
  *        ...
  * }
  *
@@ -96,27 +94,10 @@
  * Event Type
  *****************************************************************************/
 
-/* Private structure defined in misc/events.c */
-struct vlc_event_listeners_group_t;
-
-/* Event manager type */
-typedef struct vlc_event_manager_t
-{
-    void * p_obj;
-    vlc_mutex_t object_lock;
-    vlc_mutex_t event_sending_lock;
-    DECL_ARRAY(struct vlc_event_listeners_group_t *) listeners_groups;
-} vlc_event_manager_t;
-
 /* List of event */
 typedef enum vlc_event_type_t {
-    /* Input (thread) events */
-    vlc_InputStateChanged,
-    vlc_InputSelectedStreamChanged,
-
     /* Input item events */
     vlc_InputItemMetaChanged,
-    vlc_InputItemSubItemAdded,
     vlc_InputItemSubItemTreeAdded,
     vlc_InputItemDurationChanged,
     vlc_InputItemPreparsedChanged,
@@ -124,24 +105,21 @@ typedef enum vlc_event_type_t {
     vlc_InputItemInfoChanged,
     vlc_InputItemErrorWhenReadingChanged,
     vlc_InputItemPreparseEnded,
-
-    /* Service Discovery event */
-    vlc_ServicesDiscoveryItemAdded,
-    vlc_ServicesDiscoveryItemRemoved,
-    vlc_ServicesDiscoveryItemRemoveAll,
-    vlc_ServicesDiscoveryStarted,
-    vlc_ServicesDiscoveryEnded,
-
-    /* Renderer Discovery events */
-    vlc_RendererDiscoveryItemAdded,
-    vlc_RendererDiscoveryItemRemoved,
-
-    /* Addons Manager events */
-    vlc_AddonFound,
-    vlc_AddonsDiscoveryEnded,
-    vlc_AddonChanged
 } vlc_event_type_t;
 
+typedef struct vlc_event_listeners_group_t
+{
+    DECL_ARRAY(struct vlc_event_listener_t *) listeners;
+} vlc_event_listeners_group_t;
+
+/* Event manager type */
+typedef struct vlc_event_manager_t
+{
+    void * p_obj;
+    vlc_mutex_t lock;
+    vlc_event_listeners_group_t events[vlc_InputItemPreparseEnded + 1];
+} vlc_event_manager_t;
+
 /* Event definition */
 typedef struct vlc_event_t
 {
@@ -149,16 +127,6 @@ typedef struct vlc_event_t
     void * p_obj; /* Sender object, automatically filled by vlc_event_send() */
     union vlc_event_type_specific
     {
-        /* Input (thread) events */
-        struct vlc_input_state_changed
-        {
-            int new_state;
-        } input_state_changed;
-        struct vlc_input_selected_stream_changed
-        {
-            void * unused;
-        } input_selected_stream_changed;
-
         /* Input item events */
         struct vlc_input_item_meta_changed
         {
@@ -196,41 +164,6 @@ typedef struct vlc_event_t
         {
             int new_status;
         } input_item_preparse_ended;
-
-        /* Service discovery events */
-        struct vlc_services_discovery_item_added
-        {
-            input_item_t * p_new_item;
-            const char * psz_category;
-        } services_discovery_item_added;
-        struct vlc_services_discovery_item_removed
-        {
-            input_item_t * p_item;
-        } services_discovery_item_removed;
-        struct vlc_services_discovery_started
-        {
-            void * unused;
-        } services_discovery_started;
-        struct vlc_services_discovery_ended
-        {
-            void * unused;
-        } services_discovery_ended;
-
-        /* Renderer discovery events */
-        struct vlc_renderer_discovery_item_added
-        {
-            vlc_renderer_item * p_new_item;
-        } renderer_discovery_item_added;
-        struct vlc_renderer_discovery_item_removed
-        {
-            vlc_renderer_item * p_item;
-        } renderer_discovery_item_removed;
-
-        /* Addons */
-        struct vlc_addon_generic_event
-        {
-            addon_entry_t * p_entry;
-        } addon_generic_event;
     } u;
 } vlc_event_t;
 
@@ -245,23 +178,17 @@ typedef void ( *vlc_event_callback_t )( const vlc_event_t *, void * );
  * p_obj points to the object that owns the event manager, and from
  * which events are sent
  */
-VLC_API int vlc_event_manager_init( vlc_event_manager_t * p_em, void * p_obj );
+void vlc_event_manager_init( vlc_event_manager_t * p_em, void * p_obj );
 
 /*
  * Destroy
  */
-VLC_API void vlc_event_manager_fini( vlc_event_manager_t * p_em );
-
-/*
- * Tells a specific event manager that it will handle event_type object
- */
-VLC_API int vlc_event_manager_register_event_type( vlc_event_manager_t * p_em,
-                                                   vlc_event_type_t );
+void vlc_event_manager_fini( vlc_event_manager_t * p_em );
 
 /*
  * Send an event to the listener attached to this p_em.
  */
-VLC_API void vlc_event_send( vlc_event_manager_t * p_em, vlc_event_t * );
+void vlc_event_send( vlc_event_manager_t * p_em, vlc_event_t * );
 
 /*
  * Add a callback for an event.
