@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2007 Vincent Penne
  * Some code converted from ProjectX java dvb decoder (c) 2001-2005 by dvb.matt
- * $Id: 4f8842a95f4a94cb326d3e48234014852f04c235 $
+ * $Id: b0a4e773f2742f71ac735ddc74f510fa7531279a $
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -49,7 +49,7 @@
  *****************************************************************************/
 static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
-static subpicture_t *Decode( decoder_t *, block_t ** );
+static int  Decode( decoder_t *, block_t * );
 
 #define OVERRIDE_PAGE_TEXT N_("Override page")
 #define OVERRIDE_PAGE_LONGTEXT N_("Override the indicated page, try this if " \
@@ -70,7 +70,7 @@ static subpicture_t *Decode( decoder_t *, block_t ** );
 vlc_module_begin ()
     set_description( N_("Teletext subtitles decoder") )
     set_shortname( "Teletext" )
-    set_capability( "decoder", 50 )
+    set_capability( "spu decoder", 50 )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_SCODEC )
     set_callbacks( Open, Close )
@@ -179,11 +179,10 @@ static int Open( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    p_dec->pf_decode_sub = Decode;
+    p_dec->pf_decode = Decode;
     p_sys = p_dec->p_sys = calloc( 1, sizeof(*p_sys) );
     if( p_sys == NULL )
         return VLC_ENOMEM;
-    p_dec->fmt_out.i_cat = SPU_ES;
     p_dec->fmt_out.i_codec = 0;
 
     p_sys->i_align = 0;
@@ -334,9 +333,8 @@ static void decode_string( char * res, int res_len,
 {
     char utf8[7];
     char * pt = res;
-    int i;
 
-    for ( i = 0; i < len; i++ )
+    for ( int i = 0; i < len; i++ )
     {
         int in = bytereverse( packet[i] ) & 0x7f;
         uint16_t out = 32;
@@ -429,14 +427,13 @@ static void decode_string( char * res, int res_len,
 /*****************************************************************************
  * Decode:
  *****************************************************************************/
-static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
+static int Decode( decoder_t *p_dec, block_t *p_block )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
-    block_t       *p_block;
     subpicture_t  *p_spu = NULL;
     video_format_t fmt;
     /* int erase = 0; */
-    int len, offset;
+    int len;
 #if 0
     int i_wanted_magazine = i_conf_wanted_page / 100;
     int i_wanted_page = 0x10 * ((i_conf_wanted_page % 100) / 10)
@@ -445,17 +442,15 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
     bool b_update = false;
     char psz_text[512], *pt = psz_text;
     char psz_line[256];
-    int i, total;
+    int total;
 
-    if( pp_block == NULL || *pp_block == NULL )
-        return NULL;
-    p_block = *pp_block;
-    *pp_block = NULL;
+    if( p_block == NULL ) /* No Drain */
+        return VLCDEC_SUCCESS;
 
     dbg((p_dec, "start of telx packet with header %2x\n",
                 * (uint8_t *) p_block->p_buffer));
     len = p_block->i_buffer;
-    for ( offset = 1; offset + 46 <= len; offset += 46 )
+    for ( int offset = 1; offset + 46 <= len; offset += 46 )
     {
         uint8_t * packet = (uint8_t *) p_block->p_buffer+offset;
 //        int vbi = ((0x20 & packet[2]) != 0 ? 0 : 313) + (0x1F & packet[2]);
@@ -490,9 +485,8 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
         {
             /* row 0 : flags and header line */
             int flag = 0;
-            int a;
 
-            for ( a = 0; a < 6; a++ )
+            for ( int a = 0; a < 6; a++ )
             {
                 flag |= (0xF & (bytereverse( hamming_8_4(packet[8 + a]) ) >> 4))
                           << (a * 4);
@@ -545,12 +539,10 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
             if ( /*p_block->i_pts > p_sys->prev_pts + 1500000 && */
                  p_sys->b_erase[magazine] )
             {
-                int i;
-
                 dbg((p_dec, "ERASE !\n"));
 
                 p_sys->b_erase[magazine] = 0;
-                for ( i = 1; i < 32; i++ )
+                for ( int i = 1; i < 32; i++ )
                 {
                     if ( !p_sys->ppsz_lines[i][0] ) continue;
                     /* b_update = true; */
@@ -655,7 +647,7 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
         goto error;
 
     total = 0;
-    for ( i = 1; i < 24; i++ )
+    for ( int i = 1; i < 24; i++ )
     {
         size_t l = strlen( p_sys->ppsz_lines[i] );
 
@@ -716,7 +708,9 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
     dbg((p_dec, "%ld --> %ld\n", (long int) p_block->i_pts/100000, (long int)p_block->i_length/100000));
 
     block_Release( p_block );
-    return p_spu;
+    if( p_spu != NULL )
+        decoder_QueueSub( p_dec, p_spu );
+    return VLCDEC_SUCCESS;
 
 error:
     if ( p_spu != NULL )
@@ -726,5 +720,5 @@ error:
     }
 
     block_Release( p_block );
-    return NULL;
+    return VLCDEC_SUCCESS;
 }
