@@ -32,7 +32,7 @@
 #include "../http/Chunk.h"
 #include "../encryption/CommonEncryption.hpp"
 #include "../tools/Properties.hpp"
-#include "Time.hpp"
+#include "../Time.hpp"
 
 namespace adaptive
 {
@@ -47,6 +47,7 @@ namespace adaptive
     {
         class BaseRepresentation;
         class SubSegment;
+        class Segment;
         class SegmentChunk;
 
         using namespace http;
@@ -70,19 +71,16 @@ namespace adaptive
                 virtual uint64_t                        getSequenceNumber() const;
                 virtual bool                            isTemplate      () const;
                 virtual size_t                          getOffset       () const;
-                virtual std::vector<ISegment*>          subSegments     () = 0;
-                virtual void                            addSubSegment   (SubSegment *) = 0;
                 virtual void                            debug           (vlc_object_t *,int = 0) const;
                 virtual bool                            contains        (size_t byte) const;
                 virtual int                             compare         (ISegment *) const;
                 void                                    setEncryption   (CommonEncryption &);
-                int                                     getClassId      () const;
+                void                                    setDisplayTime  (mtime_t);
+                mtime_t                                 getDisplayTime  () const;
                 Property<stime_t>       startTime;
                 Property<stime_t>       duration;
                 bool                    discontinuity;
 
-                static const int CLASSID_ISEGMENT = 0;
-
             protected:
                 virtual bool                            prepareChunk    (SharedResources *,
                                                                          SegmentChunk *,
@@ -91,9 +89,9 @@ namespace adaptive
                 size_t                  startByte;
                 size_t                  endByte;
                 std::string             debugName;
-                int                     classId;
                 bool                    templated;
                 uint64_t                sequence;
+                mtime_t                displayTime;
         };
 
         class Segment : public ISegment
@@ -101,43 +99,34 @@ namespace adaptive
             public:
                 Segment( ICanonicalUrl *parent );
                 ~Segment();
-                virtual SegmentChunk* createChunk(AbstractChunkSource *, BaseRepresentation *); /* impl */
+                virtual SegmentChunk* createChunk(AbstractChunkSource *, BaseRepresentation *) override;
                 virtual void setSourceUrl( const std::string &url );
-                virtual Url getUrlSegment() const; /* impl */
-                virtual std::vector<ISegment*> subSegments();
-                virtual void debug(vlc_object_t *,int = 0) const;
+                virtual Url getUrlSegment() const override;
+                virtual const std::vector<Segment*> & subSegments() const;
+                virtual void debug(vlc_object_t *,int = 0) const override;
                 virtual void addSubSegment(SubSegment *);
-                static const int CLASSID_SEGMENT = 1;
 
             protected:
-                std::vector<SubSegment *> subsegments;
+                std::vector<Segment *> subsegments;
                 Url sourceUrl;
-                int size;
         };
 
         class InitSegment : public Segment
         {
             public:
                 InitSegment( ICanonicalUrl *parent );
-                static const int CLASSID_INITSEGMENT = 2;
         };
 
         class IndexSegment : public Segment
         {
             public:
                 IndexSegment( ICanonicalUrl *parent );
-                static const int CLASSID_INDEXSEGMENT = 3;
         };
 
-        class SubSegment : public ISegment
+        class SubSegment : public Segment
         {
             public:
-                SubSegment(ISegment *, size_t start, size_t end);
-                virtual SegmentChunk* createChunk(AbstractChunkSource *, BaseRepresentation *); /* impl */
-                virtual Url getUrlSegment() const; /* impl */
-                virtual std::vector<ISegment*> subSegments();
-                virtual void addSubSegment(SubSegment *);
-                static const int CLASSID_SUBSEGMENT = 4;
+                SubSegment(Segment *, size_t start, size_t end);
         };
     }
 }
