@@ -2,7 +2,7 @@
  * VLCFSPanelController.m: macOS fullscreen controls window controller
  *****************************************************************************
  * Copyright (C) 2006-2016 VLC authors and VideoLAN
- * $Id: 61c82f4603bb56a8073bc80eebcff3c8323587de $
+ * $Id: 036853db1ab8ec9ad891387c4d86490575a32406 $
  *
  * Authors: Jérôme Decoodt <djc at videolan dot org>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
@@ -85,6 +85,18 @@ static NSString *kAssociatedFullscreenRect = @"VLCFullscreenAssociatedWindowRect
     /* Inject correct background view depending on OS support */
     if (OSX_YOSEMITE_AND_HIGHER) {
         [self injectVisualEffectView];
+
+        // Large panel configuration (only for modern macOS versions)
+        if (var_InheritBool(getIntf(), "macosx-large-text")) {
+            NSFont *textFont = [NSFont systemFontOfSize:16.];
+
+            self.mediaTitle.font = textFont;
+            self.elapsedTime.font = textFont;
+            self.remainingOrTotalTime.font = textFont;
+
+            [_heightMaxConstraint setConstant:42. + 8.];
+        }
+
     } else {
         [self injectBackgroundView];
     }
@@ -273,17 +285,13 @@ static NSString *kAssociatedFullscreenRect = @"VLCFullscreenAssociatedWindowRect
     } else {
         [_remainingOrTotalTime setHidden:NO];
 
-        NSString *totalTime;
+        mtime_t remaining = 0;
+        if (dur > t)
+            remaining = dur - t;
+        NSString *remainingTime = [NSString stringWithFormat:@"-%s", secstotimestr(psz_time, (remaining / 1000000))];
+        NSString *totalTime = toNSStr(secstotimestr(psz_time, (dur / 1000000)));
 
-        if ([_remainingOrTotalTime timeRemaining]) {
-            mtime_t remaining = 0;
-            if (dur > t)
-                remaining = dur - t;
-            totalTime = [NSString stringWithFormat:@"-%s", secstotimestr(psz_time, (remaining / 1000000))];
-        } else {
-            totalTime = toNSStr(secstotimestr(psz_time, (dur / 1000000)));
-        }
-        [_remainingOrTotalTime setStringValue:totalTime];
+        [_remainingOrTotalTime setTime:totalTime withRemainingTime:remainingTime];
     }
 
     /* Update current position (left field) */
@@ -542,10 +550,9 @@ static NSString *kAssociatedFullscreenRect = @"VLCFullscreenAssociatedWindowRect
     [self.window setContentView:view];
     [self.window.contentView addSubview:_controlsView];
 
-    /* Disable adjusting height to workaround autolayout problems */
-    [_heightMaxConstraint setConstant:42.0];
-    [self.window setMaxSize:NSMakeSize(4068, 80)];
-    [self.window setMinSize:NSMakeSize(480, 80)];
+    // Needed on old OS to allow correct resizing of window
+    [self.window setMaxSize:NSMakeSize(4068, 84.)];
+    [self.window setMinSize:NSMakeSize(480, 84.)];
 }
 
 - (void)dealloc
