@@ -2,7 +2,7 @@
  * controller.hpp : Controller for the main interface
  ****************************************************************************
  * Copyright (C) 2006-2008 the VideoLAN team
- * $Id: dab36fdbceb6b43d260e87d28285398178610e8d $
+ * $Id: dc8bacce688ba82f47448f6e2892d0c701b726f1 $
  *
  * Authors: Jean-Baptiste Kempf <jb@videolan.org>
  *
@@ -81,7 +81,6 @@ typedef enum buttonType_e
     NEXT_BUTTON,
     OPEN_SUB_BUTTON,
     FULLWIDTH_BUTTON,
-    RENDERER_BUTTON,
     BUTTON_MAX,
 
     SPLITTER = 0x20,
@@ -111,7 +110,7 @@ static const char* const nameL[BUTTON_MAX] = { N_("Play"), N_("Stop"), N_("Open"
     N_("Record"), N_("A→B Loop"), N_("Frame By Frame"), N_("Trickplay Reverse"),
     N_("Step backward" ), N_("Step forward"), N_("Quit"), N_("Random"),
     N_("Loop / Repeat"), N_("Information"), N_("Previous"), N_("Next"),
-    N_("Open subtitles"), N_("Dock fullscreen controller"), N_("Renderer output")
+    N_("Open subtitles"), N_("Dock fullscreen controller")
 };
 static const char* const tooltipL[BUTTON_MAX] = { I_PLAY_TOOLTIP,
     N_("Stop playback"), N_("Open a medium"),
@@ -126,18 +125,16 @@ static const char* const tooltipL[BUTTON_MAX] = { I_PLAY_TOOLTIP,
     N_("Previous media in the playlist"), N_("Next media in the playlist"),
     N_("Open subtitle file"),
     N_("Dock/undock fullscreen controller to/from bottom of screen"),
-    N_("Select a Renderer device to output video/audio")
 };
-static const QString iconL[BUTTON_MAX] ={ ":/toolbar/play_b", ":/toolbar/stop_b",
-    ":/toolbar/eject", ":/toolbar/previous_b", ":/toolbar/next_b",
-    ":/toolbar/slower", ":/toolbar/faster", ":/toolbar/fullscreen",
-    ":/toolbar/defullscreen", ":/toolbar/extended", ":/toolbar/playlist",
-    ":/toolbar/snapshot", ":/toolbar/record", ":/toolbar/atob_nob",
-    ":/toolbar/frame", ":/toolbar/reverse", ":/toolbar/skip_back",
-    ":/toolbar/skip_fw", ":/toolbar/clear", ":/buttons/playlist/shuffle_on",
-    ":/buttons/playlist/repeat_all", ":/menu/info",
-    ":/toolbar/previous_b", ":/toolbar/next_b", ":/toolbar/eject", ":/toolbar/space",
-    ":/toolbar/renderer"
+static const QString iconL[BUTTON_MAX] ={ ":/toolbar/play_b.svg", ":/toolbar/stop_b.svg",
+    ":/toolbar/eject.svg", ":/toolbar/previous_b.svg", ":/toolbar/next_b.svg",
+    ":/toolbar/slower.svg", ":/toolbar/faster.svg", ":/toolbar/fullscreen.svg",
+    ":/toolbar/defullscreen.svg", ":/toolbar/extended.svg", ":/toolbar/playlist.svg",
+    ":/toolbar/snapshot.svg", ":/toolbar/record.svg", ":/toolbar/atob_nob.svg",
+    ":/toolbar/frame.svg", ":/toolbar/reverse.svg", ":/toolbar/skip_back.svg",
+    ":/toolbar/skip_fw.svg", ":/toolbar/clear.svg", ":/buttons/playlist/shuffle_on.svg",
+    ":/buttons/playlist/repeat_all.svg", ":/menu/info.svg",
+    ":/toolbar/previous_b.svg", ":/toolbar/next_b.svg", ":/toolbar/eject.svg", ":/toolbar/space.svg"
 };
 
 enum
@@ -220,6 +217,7 @@ public:
 protected:
     friend class MainInterface;
 
+    bool advancedAvailable();
     bool b_advancedVisible;
 
 private:
@@ -260,6 +258,11 @@ public:
     void toggleFullwidth();
     void updateFullwidthGeometry( int number );
     int targetScreen();
+    void setTargetScreen( int );
+
+private:
+    static int FullscreenChanged( vlc_object_t *obj,
+                    const char *, vlc_value_t, vlc_value_t new_val, void *data );
 
 signals:
     void keyPressed( QKeyEvent * );
@@ -316,6 +319,10 @@ private:
 
     bool isWideFSC;
     int i_sensitivity;
+
+#ifdef QT5_HAS_WAYLAND
+    bool b_hasWayland;
+#endif
 };
 
 #endif
