@@ -2,7 +2,7 @@
  * window_manager.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: 093e6d33930994219d119a2d382950122c30c003 $
+ * $Id: 7941cd5c6ea3999ce1c6c92a5209e0e8aa6030fe $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -325,7 +325,11 @@ void WindowManager::maximize( TopWindow &rWindow )
                                rWindow.getLeft() + rWindow.getWidth(),
                                rWindow.getTop() + rWindow.getHeight() );
 
-    SkinsRect workArea = OSFactory::instance( getIntf() )->getWorkArea();
+    // maximise the window within the current screen (multiple screens allowed)
+    int x, y, width, height;
+    rWindow.getMonitorInfo( &x, &y, &width, &height );
+    SkinsRect workArea(x, y, x + width, y + height);
+
     // Move the window
     startMove( rWindow );
     move( rWindow, workArea.getLeft(), workArea.getTop() );
