@@ -2,7 +2,7 @@
  * win32_dragdrop.cpp
  *****************************************************************************
  * Copyright (C) 2003 the VideoLAN team
- * $Id: 098ef65194306bbdef8dc0e249e707e6e6f93ab9 $
+ * $Id: bb936ac6f91c57c1656e776cbe7982b1939537fd $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulière <ipkiss@via.ecp.fr>
@@ -24,7 +24,7 @@
 
 #ifdef WIN32_SKINS
 
-#include "win32/win32_dragdrop.hpp"
+#include "win32_dragdrop.hpp"
 #include "../commands/cmd_add_item.hpp"
 #include "../events/evt_dragndrop.hpp"
 #include <list>
