@@ -2,7 +2,7 @@
  * adjust.c : Contrast/Hue/Saturation/Brightness video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2006 VLC authors and VideoLAN
- * $Id: ea50221c183f5531efc78790f7d4af74ccdd3ec5 $
+ * $Id: fc12d666402bafcaa9ea1ac13d6696ddc578953a $
  *
  * Authors: Simon Latapie <garf@via.ecp.fr>
  *          Antoine Cellerier <dionoea -at- videolan d0t org>
@@ -34,9 +34,10 @@
 #include <math.h>
 
 #include <vlc_common.h>
+#include <vlc_atomic.h>
 #include <vlc_plugin.h>
-
 #include <vlc_filter.h>
+#include <vlc_picture.h>
 #include "filter_picture.h"
 
 #include "adjust_sat_hue.h"
@@ -63,8 +64,8 @@ static int AdjustCallback( vlc_object_t *p_this, char const *psz_var,
         "defined below." )
 #define CONT_TEXT N_("Image contrast (0-2)")
 #define CONT_LONGTEXT N_("Set the image contrast, between 0 and 2. Defaults to 1.")
-#define HUE_TEXT N_("Image hue (0-360)")
-#define HUE_LONGTEXT N_("Set the image hue, between 0 and 360. Defaults to 0.")
+#define HUE_TEXT N_("Image hue (-180..180)")
+#define HUE_LONGTEXT N_("Set the image hue, between -180 and 180. Defaults to 0.")
 #define SAT_TEXT N_("Image saturation (0-3)")
 #define SAT_LONGTEXT N_("Set the image saturation, between 0 and 3. Defaults to 1.")
 #define LUM_TEXT N_("Image brightness (0-2)")
@@ -77,7 +78,7 @@ vlc_module_begin ()
     set_shortname( N_("Image adjust" ))
     set_category( CAT_VIDEO )
     set_subcategory( SUBCAT_VIDEO_VFILTER )
-    set_capability( "video filter2", 0 )
+    set_capability( "video filter", 0 )
 
     add_float_with_range( "contrast", 1.0, 0.0, 2.0,
                           CONT_TEXT, CONT_LONGTEXT, false )
@@ -112,13 +113,12 @@ static const char *const ppsz_filter_options[] = {
  *****************************************************************************/
 struct filter_sys_t
 {
-    vlc_mutex_t lock;
-    float f_contrast;
-    float f_brightness;
-    float f_hue;
-    float f_saturation;
-    float f_gamma;
-    bool  b_brightness_threshold;
+    vlc_atomic_float f_contrast;
+    vlc_atomic_float f_brightness;
+    vlc_atomic_float f_hue;
+    vlc_atomic_float f_saturation;
+    vlc_atomic_float f_gamma;
+    atomic_bool  b_brightness_threshold;
     int (*pf_process_sat_hue)( picture_t *, picture_t *, int, int, int,
                                int, int );
     int (*pf_process_sat_hue_clip)( picture_t *, picture_t *, int, int,
@@ -145,19 +145,6 @@ static int Create( vlc_object_t *p_this )
         return VLC_ENOMEM;
     p_sys = p_filter->p_sys;
 
-    /* needed to get options passed in transcode using the
-     * adjust{name=value} syntax */
-    config_ChainParse( p_filter, "", ppsz_filter_options,
-                   p_filter->p_cfg );
-
-    p_sys->f_contrast = var_CreateGetFloatCommand( p_filter, "contrast" );
-    p_sys->f_brightness = var_CreateGetFloatCommand( p_filter, "brightness" );
-    p_sys->f_hue = var_CreateGetFloatCommand( p_filter, "hue" );
-    p_sys->f_saturation = var_CreateGetFloatCommand( p_filter, "saturation" );
-    p_sys->f_gamma = var_CreateGetFloatCommand( p_filter, "gamma" );
-    p_sys->b_brightness_threshold =
-        var_CreateGetBoolCommand( p_filter, "brightness-threshold" );
-
     /* Choose Planar/Packed function and pointer to a Hue/Saturation processing
      * function*/
     switch( p_filter->fmt_in.video.i_chroma )
@@ -185,12 +172,29 @@ static int Create( vlc_object_t *p_this )
             break;
 
         default:
-            msg_Err( p_filter, "Unsupported input chroma (%4.4s)",
+            msg_Dbg( p_filter, "Unsupported input chroma (%4.4s)",
                      (char*)&(p_filter->fmt_in.video.i_chroma) );
+            free(p_sys);
             return VLC_EGENERIC;
     }
 
-    vlc_mutex_init( &p_sys->lock );
+    /* needed to get options passed in transcode using the
+     * adjust{name=value} syntax */
+    config_ChainParse( p_filter, "", ppsz_filter_options, p_filter->p_cfg );
+
+    vlc_atomic_init_float( &p_sys->f_contrast,
+                           var_CreateGetFloatCommand( p_filter, "contrast" ) );
+    vlc_atomic_init_float( &p_sys->f_brightness,
+                           var_CreateGetFloatCommand( p_filter, "brightness" ) );
+    vlc_atomic_init_float( &p_sys->f_hue,
+                           var_CreateGetFloatCommand( p_filter, "hue" ) );
+    vlc_atomic_init_float( &p_sys->f_saturation,
+                           var_CreateGetFloatCommand( p_filter, "saturation" ) );
+    vlc_atomic_init_float( &p_sys->f_gamma,
+                           var_CreateGetFloatCommand( p_filter, "gamma" ) );
+    atomic_init( &p_sys->b_brightness_threshold,
+                 var_CreateGetBoolCommand( p_filter, "brightness-threshold" ) );
+
     var_AddCallback( p_filter, "contrast",   AdjustCallback, p_sys );
     var_AddCallback( p_filter, "brightness", AdjustCallback, p_sys );
     var_AddCallback( p_filter, "hue",        AdjustCallback, p_sys );
@@ -218,7 +222,6 @@ static void Destroy( vlc_object_t *p_this )
     var_DelCallback( p_filter, "brightness-threshold",
                                              AdjustCallback, p_sys );
 
-    vlc_mutex_destroy( &p_sys->lock );
     free( p_sys );
 }
 
@@ -268,19 +271,16 @@ static picture_t *FilterPlanar( filter_t *p_filter, picture_t *p_pic )
     const unsigned i_mid = i_range >> 1;
 
     /* Get variables */
-    vlc_mutex_lock( &p_sys->lock );
-    int32_t i_cont = lroundf( p_sys->f_contrast * f_max );
-    int32_t i_lum = lroundf( (p_sys->f_brightness - 1.f) * f_max );
-    float f_hue = p_sys->f_hue * (float)(M_PI / 180.);
-    int i_sat = (int)( p_sys->f_saturation * f_range );
-    float f_gamma = 1.f / p_sys->f_gamma;
-    bool b_thres = p_sys->b_brightness_threshold;
-    vlc_mutex_unlock( &p_sys->lock );
+    int32_t i_cont = lroundf( vlc_atomic_load_float( &p_sys->f_contrast ) * f_max );
+    int32_t i_lum = lroundf( (vlc_atomic_load_float( &p_sys->f_brightness ) - 1.f) * f_max );
+    float f_hue = vlc_atomic_load_float( &p_sys->f_hue ) * (float)(M_PI / 180.);
+    int i_sat = (int)( vlc_atomic_load_float( &p_sys->f_saturation ) * f_range );
+    float f_gamma = 1.f / vlc_atomic_load_float( &p_sys->f_gamma );
 
     /*
      * Threshold mode drops out everything about luma, contrast and gamma.
      */
-    if( !b_thres )
+    if( !atomic_load( &p_sys->b_brightness_threshold ) )
     {
 
         /* Contrast is a fast but kludged function, so I put this gap to be
@@ -436,7 +436,6 @@ static picture_t *FilterPacked( filter_t *p_filter, picture_t *p_pic )
 
     int i_pitch, i_visible_pitch;
 
-    bool b_thres;
     double  f_hue;
     double  f_gamma;
     int32_t i_cont, i_lum;
@@ -469,19 +468,16 @@ static picture_t *FilterPacked( filter_t *p_filter, picture_t *p_pic )
     }
 
     /* Get variables */
-    vlc_mutex_lock( &p_sys->lock );
-    i_cont = (int)( p_sys->f_contrast * 255 );
-    i_lum = (int)( (p_sys->f_brightness - 1.0)*255 );
-    f_hue = p_sys->f_hue * (float)(M_PI / 180.);
-    i_sat = (int)( p_sys->f_saturation * 256 );
-    f_gamma = 1.0 / p_sys->f_gamma;
-    b_thres = p_sys->b_brightness_threshold;
-    vlc_mutex_unlock( &p_sys->lock );
+    i_cont = (int)( vlc_atomic_load_float( &p_sys->f_contrast ) * 255 );
+    i_lum = (int)( (vlc_atomic_load_float( &p_sys->f_brightness ) - 1.0)*255 );
+    f_hue = vlc_atomic_load_float( &p_sys->f_hue ) * (float)(M_PI / 180.);
+    i_sat = (int)( vlc_atomic_load_float( &p_sys->f_saturation ) * 256 );
+    f_gamma = 1.0 / vlc_atomic_load_float( &p_sys->f_gamma );
 
     /*
      * Threshold mode drops out everything about luma, contrast and gamma.
      */
-    if( !b_thres )
+    if( !atomic_load( &p_sys->b_brightness_threshold ) )
     {
 
         /* Contrast is a fast but kludged function, so I put this gap to be
@@ -601,20 +597,18 @@ static int AdjustCallback( vlc_object_t *p_this, char const *psz_var,
     VLC_UNUSED(p_this); VLC_UNUSED(oldval);
     filter_sys_t *p_sys = (filter_sys_t *)p_data;
 
-    vlc_mutex_lock( &p_sys->lock );
     if( !strcmp( psz_var, "contrast" ) )
-        p_sys->f_contrast = newval.f_float;
+        vlc_atomic_store_float( &p_sys->f_contrast, newval.f_float );
     else if( !strcmp( psz_var, "brightness" ) )
-        p_sys->f_brightness = newval.f_float;
+        vlc_atomic_store_float( &p_sys->f_brightness, newval.f_float );
     else if( !strcmp( psz_var, "hue" ) )
-        p_sys->f_hue = newval.f_float;
+        vlc_atomic_store_float( &p_sys->f_hue, newval.f_float );
     else if( !strcmp( psz_var, "saturation" ) )
-        p_sys->f_saturation = newval.f_float;
+        vlc_atomic_store_float( &p_sys->f_saturation, newval.f_float );
     else if( !strcmp( psz_var, "gamma" ) )
-        p_sys->f_gamma = newval.f_float;
+        vlc_atomic_store_float( &p_sys->f_gamma, newval.f_float );
     else if( !strcmp( psz_var, "brightness-threshold" ) )
-        p_sys->b_brightness_threshold = newval.b_bool;
-    vlc_mutex_unlock( &p_sys->lock );
+        atomic_store( &p_sys->b_brightness_threshold, newval.b_bool );
 
     return VLC_SUCCESS;
 }
