@@ -2,7 +2,7 @@
  * playlist_fetcher.h:
  *****************************************************************************
  * Copyright (C) 1999-2008 VLC authors and VideoLAN
- * $Id: c5d9a3ddbbbb23f1c923178bbc59cfb2e0e946d1 $
+ * $Id: 06718acf77542c69d2f881ad5ed8662d693fe2fb $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Clément Stenac <zorglub@videolan.org>
@@ -46,8 +46,8 @@ playlist_fetcher_t *playlist_fetcher_New( vlc_object_t * );
  * The input item is retained until the art fetching is done or until the
  * fetcher object is destroyed.
  */
-void playlist_fetcher_Push( playlist_fetcher_t *, input_item_t *,
-                            input_item_meta_request_option_t );
+int playlist_fetcher_Push( playlist_fetcher_t *, input_item_t *,
+                           input_item_meta_request_option_t, int );
 
 /**
  * This function destroys the fetcher object and thread.
